#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QFileDialog>
#include <QThread>
#include <QTimer>

#include "wrappers.h"
#include "trackerworker.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    backendPid(0),
    backendPipe(-1)
{
    ui->setupUi(this);

    /* Create a thread for workers */
    QThread* workerThread = new QThread(this);

    /* Create a worker to update the tracker */
    TrackerWorker * trackerWorker = new TrackerWorker();

    /* Move it to the worker thread. This means that slots of this worker
     * will run in the worker thread, and not block the UI */
    trackerWorker->moveToThread(workerThread);

    /* Connect tracker worker */
    connect(trackerWorker, SIGNAL (finished(float, float, float, float, float, float)),
            this, SLOT (updateTracker(float, float, float, float, float, float)));

    /* Connect and disconnect from backend when signals emitted */
    connect(this, SIGNAL (connectWorkers()), trackerWorker, SLOT (connectBackend()));
    connect(this, SIGNAL (disconnectWorkers()), trackerWorker, SLOT (disconnectBackend()));

    /* Create other workers and add them to the worker thread, then connect them */

    /* Now we can activate the slots of the workers with impunity and not block the UI thread */
    /* Connect refresh button and refresh timer to tracker worker */
    QTimer * trackerTimer = new QTimer(this);
    connect(trackerTimer, SIGNAL(timeout()), trackerWorker, SLOT(process()));
    connect(findChild<QPushButton *>("pbRefresh"), SIGNAL (clicked()), trackerWorker, SLOT (process()));

    /* Start the things */
    trackerTimer->start(300);
    workerThread->start();

    /* Create a timer to poll stdout and populate virtual console */
    QTimer * consoleTimer = new QTimer(this);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::updateConsole()
{
    if (backendPipe != -1) {
        char buf[256];
        size_t len = 0;
        len = readBackend(backendPipe, buf, len);
        printf("%d\n", len);
        if (len > 0) {
            printf("From pipe: %s", buf);
            QLineEdit * con = findChild<QLineEdit *>("vConsole");
            con->setText(con->text().append(buf));
        }
    }
}

void MainWindow::updateTracker(float x, float y, float z, float p, float r, float yaw)
{
    findChild<QLineEdit *>("xLineEdit")->setText(QString::number(x));
    findChild<QLineEdit *>("yLineEdit")->setText(QString::number(y));
    findChild<QLineEdit *>("zLineEdit")->setText(QString::number(z));
    findChild<QLineEdit *>("pLineEdit")->setText(QString::number(p));
    findChild<QLineEdit *>("rLineEdit")->setText(QString::number(r));
    findChild<QLineEdit *>("yLineEdit_2")->setText(QString::number(yaw));
}

void MainWindow::on_pbStart_clicked()
{
    this->backendPid = startBackend(findChild<QLineEdit *>("backendPath")->text().toStdString().c_str(), &backendPipe);
    findChild<QPushButton *>("pbStart")->setEnabled(false);
    findChild<QPushButton *>("pbStop")->setEnabled(true);
    backendState = 1;
}

void MainWindow::on_pbConnect_clicked()
{
    findChild<QPushButton *>("pbStart")->setEnabled(false);
    findChild<QPushButton *>("pbConnect")->setEnabled(false);
    findChild<QPushButton *>("pbStop")->setEnabled(true);
    emit(connectWorkers());
    backendState = 1;
}

void MainWindow::on_pbStop_clicked()
{
    emit(disconnectWorkers());
    if (backendPid) {
        stopBackend(backendPid, backendPipe);
        backendPipe = -1;
        backendPid = 0;
    }
    findChild<QPushButton *>("pbStart")->setEnabled(true);
    findChild<QPushButton *>("pbConnect")->setEnabled(true);
    findChild<QPushButton *>("pbStop")->setEnabled(false);
    backendState = 0;
}

void MainWindow::on_chooseBackend_clicked()
{
    QString backendPath = QFileDialog::getOpenFileName(this,
         tr("Path to Backend Executable"));
    findChild<QLineEdit *>("backendPath")->setText(backendPath);
}
